// file    : reflect.cs
// compile : csc reflect.cs

using System;
using System.Reflection;
using System.Text.RegularExpressions;

public class Person {

  public Person(string firstName, string lastName) {
    FirstName = firstName;
    LastName = lastName;
  }

  public void DisplayFirstName() {
    Console.WriteLine("First Name={0}", FirstName);
  }

  public void DisplayLastName() {
    Console.WriteLine("Last Name={0}", LastName);
  }

  public void ShowAll() {
    Console.WriteLine("Person...");
    Console.WriteLine("First Name={0}", FirstName);
    Console.WriteLine("Last Name={0}", LastName);
  }

  public readonly string FirstName;
  public readonly string LastName;
}


public class Reflect {
  public static void Main() {

    // get the Person type...
    Type typ = Type.GetType("Person");

    // create array of constructor arguments...
    object[] args = {"Joe", "Bloggs"};

    // creat an instance of the Person type...
    Object obj = Activator.CreateInstance(typ, args);

    // get a list of Person methods...
    MethodInfo[] met = typ.GetMethods();

    // find and invoke all display methods...
    Regex r = new Regex("^display", RegexOptions.IgnoreCase);
    foreach (MethodInfo m in met)
      if (r.IsMatch(m.Name))
        m.Invoke(obj, null); // invoke display method
  }
}
