// file    : ConPok.cs
// compile : csc /r:poker.dll conpok.cs

using System;
using System.Collections;
using System.Text;

using Poker;

class ConPok {

  public ConPok() {

    greeting();

    machine = Machine.Instance;
    uiBet = minBet = machine.MinBet;
    maxBet = machine.MaxBet;
    uiCredits = machine.StartCredits;

    // play until our opponent is wiped out!
    while (uiCredits >= minBet)
      nextGame();
    Console.WriteLine("*** Loser! ***  :-)");
  }

  private void greeting() {
    Console.WriteLine(
      "\nWelcome to the Console Version of Video Poker.");
    Console.WriteLine(
      "Cards are numbered 1 to 5 from left to right.");
    Console.WriteLine(
      "To hold cards, enter card numbers and hit enter.");
    Console.WriteLine(
      "Hit Ctrl-c at any time to abort.\n");
  }


  private void nextGame() {

    Console.WriteLine("Credits Remaining : {0}", uiCredits);
    Console.Write("Enter amount of bet : ");

    string reply = Console.ReadLine();
    int newBet;
    try {
      newBet = Int32.Parse(reply);
    }
    catch (Exception) {
      // use previous bet...
      newBet = uiBet;
    }

    Bet bet = new Bet(newBet, uiCredits, minBet, maxBet);
    uiBet = bet.Amount;
    uiCredits = bet.Credits;
    if (bet.Message.Equals(""))
      Console.WriteLine("Betting {0}...", uiBet);
    else
      Console.WriteLine(bet.Message + '\u0007'); // ring bell

    Hand dealHand = machine.Deal(); // deal
    Console.WriteLine("{0}", dealHand);

    Console.Write("Enter card numbers (1 to 5) to hold: ");
    string holdCards = Console.ReadLine();

    Hand drawHand =
      machine.Draw(dealHand.ToString(), holdCards, uiBet);

    int uiWin = drawHand.Score * uiBet;
    uiCredits += uiWin;
    string uiMsg =
      drawHand.ToString() + " - " + drawHand.Title + " " +
      "(Score=" + drawHand.Score + ", " +
      "Bet=" + uiBet + ", " + "Win=" + uiWin + ")";

    Console.WriteLine(uiMsg);

    Console.WriteLine(machine.Stats);
  }

  private Machine machine;
  private int minBet;
  private int maxBet;
  private int uiCredits;
  private int uiBet;

} // end Class ConPok

// action starts here...
public class Go {
  public static void Main() {
    ConPok cp = new ConPok();
  }
}