//  file:          rempok.cs
//  description:   a remote poker machine client
//  compile:       csc  /r:rempokservice.exe rempok.cs


namespace Poker {

  using System;
  using System.Runtime.Remoting;
  using System.Runtime.Remoting.Channels;
  using System.Runtime.Remoting.Channels.Tcp;
  using System.Runtime.Remoting.Channels.Http;
  using System.Runtime.Remoting.Activation;


  class RemPok {

    public static void Main(string[] args) {

      string argLine = String.Join(" ", args).ToLower() + " ";

      string uri = "";
      if (argLine.IndexOf("/tcp") >= 0) {
        Console.WriteLine("starting TCP RemPok client...");
        ChannelServices.RegisterChannel(new TcpChannel());
        uri += "tcp://localhost:6789";
      } else {
        Console.WriteLine("starting HTTP RemPok client...");
        ChannelServices.RegisterChannel(new HttpChannel());
        uri += "http://localhost:8085";
      }

      if (argLine.IndexOf("/ca") >= 0) {
        Console.WriteLine("client activation...");
      } else {
        Console.WriteLine("server activation...");
        uri += "/RemPokService"; // append well-known endpoint name
      }

      new RemPok(argLine, uri); // start game
    }

    public RemPok(string argLine, string uri) {
      this.argLine = argLine;
      this.uri = uri;
      Console.WriteLine("A remote poker game...");
      Console.WriteLine("Hit Ctrl-c at any time to abort.\n");
      while (true) nextGame(); // play
    }

    private void nextGame() {
      RemPokService service = getService();
      string dealHand = service.Deal(); // deal hand
      Console.WriteLine(dealHand); // display it
      Console.Write("Enter card numbers (1 to 5) to hold: ");
      string holdCards = Console.ReadLine();

      // draw replacement cards...
      GameResult res = service.Draw(dealHand, holdCards);
      Console.WriteLine(res.Hand);
      Console.WriteLine(res.Title);
      Console.WriteLine("Score = {0}\n", res.Score);
    }

    private RemPokService getService() {

      if (argLine.IndexOf("/ca") >= 0) {
        object[] constructorArguments = new object[0];
        object[] activationAttributes = new object[1];
        activationAttributes[0] = new UrlAttribute(uri);
        return (RemPokService)Activator.CreateInstance(
          typeof(RemPokService),
          constructorArguments,
          activationAttributes
        );
      } else {
        return (RemPokService)Activator.GetObject(
          typeof(Poker.RemPokService),
          uri
        );
      }
    }

    private string argLine = null;
    private string uri = null;
  }
}