// file    : RemPokService.cs
// compile : csc /r:poker.dll RemPokService.cs

namespace Poker {

  using System;
  using System.Runtime.Remoting;

  // use serializable GameResult struct to return game result...
  [Serializable]
  public struct GameResult {
    public string Hand;
    public int Score;
    public string Title;
  }

  public class RemPokService : MarshalByRefObject {

    public RemPokService() {
      Console.WriteLine("RemPokService activated...");
    }

    public string Deal() {
      string hand = new SimpleMachine().Deal().Text;
      Console.WriteLine("Dealing : {0}", hand);
      return hand;
    }

    public GameResult Draw(string oldHand, string holdCards) {
      GameResult g = new GameResult();
      Hand h = new SimpleMachine().Draw(oldHand, holdCards);
      g.Hand = h.Text;
      g.Score = h.Score;
      g.Title = h.Title;
      Console.WriteLine("Drawing : {0}  ({1})", g.Hand, g.Title);
      return g;
    }

    public static void Main(string[] args) {

      // get the default application configuration file name...
      string configFile =
        AppDomain.CurrentDomain.SetupInformation.ConfigurationFile;

      // configure remoting...
      Console.WriteLine("using " + configFile + "...");
      RemotingConfiguration.Configure(configFile);

      Console.WriteLine("waiting for remote calls...");
      Console.WriteLine("hit ENTER to exit...");
      Console.ReadLine();
    }
  }
}