// file:    SvcPokService.cs
// compile: csc /r:poker.dll SvcPokService.cs

namespace Poker {

  using System;
  using System.ServiceProcess;
  using System.Runtime.Remoting;
  using System.Configuration.Install;
  using System.ComponentModel;
  using System.Runtime.Remoting.Channels;
  using System.Runtime.Remoting.Channels.Http;

  // use serializable GameResult struct to return game result...
  [Serializable]
  public struct GameResult {
    public string Hand;
    public int Score;
    public string Title;
  }

  public class SvcPokService : MarshalByRefObject {

    public string Deal() {
      string hand = new SimpleMachine().Deal().Text;
      Console.WriteLine("Dealing : {0}", hand);
      return hand;
    }

    public GameResult Draw(string oldHand, string holdCards) {
      GameResult g = new GameResult();
      Hand h = new SimpleMachine().Draw(oldHand, holdCards);
      g.Hand = h.Text;
      g.Score = h.Score;
      g.Title = h.Title;
      Console.WriteLine("Drawing : {0}  ({1})", g.Hand, g.Title);
      return g;
    }
  }

  public class PokerService : ServiceBase {

    public PokerService() {
      CanPauseAndContinue = false;
      CanStop = true;
      ServiceName = "PokerService";
    }

    protected override void OnStart(string[] args) {
      ChannelServices.RegisterChannel(new HttpChannel(6789));

      RemotingConfiguration.RegisterWellKnownServiceType(
        new SvcPokService().GetType(),      // remote object type
        "SvcPokService",                    // remote object name
        WellKnownObjectMode.Singleton);     // activation mode
    }

    public static void Main() {
      ServiceBase.Run(new PokerService());
    }
  }

  [RunInstallerAttribute(true)]
  public class PokerServiceInstaller : Installer {

    public PokerServiceInstaller() {

      ServiceProcessInstaller processInstaller =
        new ServiceProcessInstaller();
      processInstaller.Account = ServiceAccount.LocalSystem;

      ServiceInstaller serviceInstaller = new ServiceInstaller();
      serviceInstaller.StartType = ServiceStartMode.Manual;
      serviceInstaller.ServiceName = "PokerService";

      Installers.Add(serviceInstaller);
      Installers.Add(processInstaller);
    }
  }
}