
// file    : WinPok.cs
// compile : csc /r:poker.dll
//           /t:winexe
//           /win32icon:poker.ico
//          winpok.cs

namespace Poker {

  using System;
  using System.Runtime.InteropServices; // for API MessageBeep
  using System.Windows.Forms;
  using System.Threading;
  using System.Drawing;
  using System.ComponentModel;

  public class WinPokForm : Form {

    public static void Main() {
      // start the Windows message loop...
      Application.Run(new WinPokForm());
    }

    public WinPokForm() {
      initUI(); // create GUI controls
      newGame(); // init poker machine, user credits, etc.
    }

    private void initUI() {
      initForm();
      initMenu();
      initStartOverButton();
      initCredits();
      initMessage();
      initBet();
      initHoldCheckBoxes();
      initDealDrawButton();
      initPayoutTable();
      initMachineStats();
      initStatusBar();
      initCards();
    }

    private void initForm() {
      // initialize the form...

      // set title bar...
      Text = ".NET Video Poker  -  The Windows Forms Version";

      // set form height and width...
      Height = 510;
      Width= 445;

      // center form and disallow resizing...
      CenterToScreen();
      MaximizeBox = false;
      FormBorderStyle = FormBorderStyle.FixedDialog;

      // set the form icon...
      Icon = getIcon("poker");
    }

    private void initMenu() {
      // initialize the menu...

      // create the form's main menu...
      Menu = new MainMenu();

      // create the File menu...
      MenuItem fileMenuItem = Menu.MenuItems.Add("&File");

      startOverMenuItem = new MenuItem(
        "&Start Over",
        new EventHandler(startOverHandler),
        Shortcut.CtrlS);
      fileMenuItem.MenuItems.Add(startOverMenuItem);

      MenuItem quitMenuItem = new MenuItem(
        "&Quit",
        new EventHandler(quitHandler),
        Shortcut.CtrlQ);
      fileMenuItem.MenuItems.Add(quitMenuItem);

      // create the Help menu...
      MenuItem helpMenuItem = Menu.MenuItems.Add("&Help");

      MenuItem aboutMenuItem = new MenuItem(
        "&About",
        new EventHandler(aboutHandler),
        Shortcut.CtrlA);
      helpMenuItem.MenuItems.Add(aboutMenuItem);
    }

    private void initStartOverButton() {
      startOverButton = new Button();
      startOverButton.Location = new Point(8, 8);
      startOverButton.Size = new Size(424, 24);
      startOverButton.Text = "&Start Over";
      startOverButton.Font =
        new Font("Verdana", 10f, FontStyle.Bold);
      startOverButton.Click +=
        new EventHandler(startOverHandler);
      Controls.Add(startOverButton);
    }

    private void initCredits() {
      // display how many credits remaining...

      Label l = new Label();
      l.Location = new Point(8, 40);
      l.Text = "CREDITS";
      l.Size = new Size(88, 24);
      l.Font = new Font("Verdana", 10f, FontStyle.Bold);
      l.TextAlign = ContentAlignment.MiddleCenter;
      Controls.Add(l);

      creditsLabel = new Label();
      creditsLabel.Location = new Point(18, 64);
      creditsLabel.Size = new Size(60, 24);
      creditsLabel.Font = new Font("Verdana", 10f, FontStyle.Bold);
      creditsLabel.BorderStyle =
        System.Windows.Forms.BorderStyle.Fixed3D;
      creditsLabel.TextAlign = ContentAlignment.MiddleCenter;
      Controls.Add(creditsLabel);
    }

    private void initMessage() {

      Label l = new Label();
      l.Text = ".NET Video Poker";
      l.Font = new Font("Verdana", 10f, FontStyle.Bold);
      l.Location = new Point(104, 40);
      l.Size = new Size(232, 24);
      l.TextAlign = ContentAlignment.MiddleCenter;
      Controls.Add(l);

      // message to the player...
      messageLabel = new Label();
      messageLabel.Font = new Font("Verdana",10f, FontStyle.Bold);
      messageLabel.Location = new Point(104, 64);
      messageLabel.Size = new Size(232, 24);
      messageLabel.TextAlign = ContentAlignment.MiddleCenter;
      Controls.Add(messageLabel);
    }

    private void initBet() {

      Label l = new Label();
      l.Text = "BET";
      l.Location = new Point(344, 40);
      l.Size = new Size(88, 24);
      l.Font = new Font("Verdana",10f, FontStyle.Bold);
      l.TextAlign = ContentAlignment.MiddleCenter;
      Controls.Add(l);

      betTextBox = new TextBox();
      betTextBox.Location = new Point(368, 64);
      betTextBox.MaxLength = 1;
      betTextBox.Font = new Font("Verdana",10f, FontStyle.Bold);
      betTextBox.Size = new Size(32, 22);
      betTextBox.TextAlign = HorizontalAlignment.Center;
      betTextBox.TabStop = false;
      betTextBox.TextChanged += new EventHandler(betChangedHandler);
      Controls.Add(betTextBox);
    }

    private void initCards() {

      card1 = new PictureBox();
      card1.Location = new Point(8, 104);
      card1.Size = new Size(72, 96);
      Controls.Add(card1);

      card2 = new PictureBox();
      card2.Location = new Point(96, 104);
      card2.Size = new Size(72, 96);
      Controls.Add(card2);

      card3 = new PictureBox();
      card3.Location = new Point(184, 104);
      card3.Size = new Size(72, 96);
      Controls.Add(card3);

      card4 = new PictureBox();
      card4.Location = new Point(272, 104);
      card4.Size = new Size(72, 96);
      Controls.Add(card4);

      card5 = new PictureBox();
      card5.Location = new Point(360, 104);
      card5.Size = new Size(72, 96);
      Controls.Add(card5);
    }

    private void initHoldCheckBoxes() {
      // init hold CheckBoxes...

      hold1 = new CheckBox();
      hold1.Location = new Point(12, 208);

      hold2 = new CheckBox();
      hold2.Location = new Point(100, 208);

      hold3 = new CheckBox();
      hold3.Location = new Point(188, 208);

      hold4 = new CheckBox();
      hold4.Location = new Point(276, 208);

      hold5 = new CheckBox();
      hold5.Location = new Point(364, 208);

      // set common HOLD checkbox attributes...
      hold1.Text = hold2.Text = hold3.Text =
        hold4.Text = hold5.Text = "HOLD";
      hold1.Font = hold2.Font = hold3.Font =
        hold4.Font = hold5.Font =
          new Font("Verdana", 11f, FontStyle.Bold);
      hold1.Size = hold2.Size = hold3.Size =
        hold4.Size = hold5.Size = new Size(80, 24);
      hold1.TextAlign = hold2.TextAlign = hold3.TextAlign =
        hold4.TextAlign = hold5.TextAlign =
          ContentAlignment.MiddleLeft;

      // add the HOLD checkboxes to the UI...
      Controls.Add(hold1);
      Controls.Add(hold2);
      Controls.Add(hold3);
      Controls.Add(hold4);
      Controls.Add(hold5);

    }

    private void initDealDrawButton() {
      dealDrawButton = new Button();
      dealDrawButton.Location = new Point(168, 240);
      dealDrawButton.Size = new Size(104, 24);
      dealDrawButton.Font =
        new Font("Verdana",10f, FontStyle.Bold);
      dealDrawButton.Click +=
        new EventHandler(dealDrawHandler);
      Controls.Add(dealDrawButton);
    }

    private void initPayoutTable() {

      // frame the payout table...
      GroupBox g = new GroupBox();
      g.Location = new Point(8, 272);
      g.Size = new Size(200, 168);
      Controls.Add(g);

      Label l = new Label();
      l.Location = new Point(5, 10);
      l.Text = Machine.PayoutTable; // payout text never changes
      l.Size = new Size(180, 150);
      l.Font =
        new Font(FontFamily.GenericMonospace, 8f, FontStyle.Bold);
      g.Controls.Add(l);
    }

    private void initMachineStats() {

      GroupBox g = new GroupBox();
      g.Location = new Point(216, 272);
      g.Size = new Size(216, 168);
      Controls.Add(g);

      machineStatsLabel = new Label();
      machineStatsLabel.Location = new Point(5, 10);
      machineStatsLabel.Size = new Size(190, 150);
      machineStatsLabel.Font =
        new Font(FontFamily.GenericMonospace, 8f, FontStyle.Bold);
      g.Controls.Add(machineStatsLabel);
    }

    private void initStatusBar() {

      statusBarPanel = new StatusBarPanel();
      statusBarPanel.BorderStyle =
        StatusBarPanelBorderStyle.Sunken;
      statusBarPanel.AutoSize =  StatusBarPanelAutoSize.Spring;
      statusBarPanel.Alignment = HorizontalAlignment.Center;

      StatusBar s = new StatusBar();
      s.ShowPanels = true;
      s.Font = new Font("Verdana", 8f, FontStyle.Bold);
      s.Panels.AddRange(new StatusBarPanel[]{statusBarPanel});
      Controls.Add(s);
    }

    private void initPokerMachine() {
      // initialize the poker machine...
      machine = Machine.Instance;
      uiBet = machine.MinBet;
      uiCredits = machine.StartCredits;
    }

    protected override void OnClosing(CancelEventArgs e) {
      base.OnClosing(e);
      // make sure the player really wants to quit...
      DialogResult r = MessageBox.Show(
          "Quit?",
          "Closing",
          MessageBoxButtons.YesNo, MessageBoxIcon.Question);
      if (r != DialogResult.Yes) e.Cancel = true;
    }

    private void startOverHandler(object sender, EventArgs e) {
      // user selected "Start Over" from the File menu...
      newGame();
    }

    private void quitHandler(object sender, EventArgs e) {
      // user selected "Quit" from the File menu...
      Close(); // close this form
    }

    private void aboutHandler(object sender, EventArgs e) {
      // user selected "About" from the Help menu...
      string msg = ".NET Video Poker - Windows Forms Version\n";
      msg += "by Fergal Grimes\n";
      MessageBox.Show(
        msg,
        ".NET Video Poker",
        MessageBoxButtons.OK,
        MessageBoxIcon.Exclamation);
    }

    private void dealDrawHandler(object sender, EventArgs e) {
      if (dealDrawButton.Text == "&DEAL")
        deal();
      else
        draw();
    }

    private void betChangedHandler(object sender, EventArgs e) {

      int newBet;
      try {
        newBet = Int32.Parse(betTextBox.Text);
      }
      catch (Exception) {
        // use previous bet...
        beep(); // alert player
        showStatus("Error: Illegal bet!");
        newBet = uiBet;
      }
      betTextBox.Text = getBet(newBet).ToString();
    }

    private int getBet(int newBet) {
      Bet bet =
        new Bet(newBet,uiCredits, machine.MinBet, machine.MaxBet);
      if (bet.Amount != newBet) {
        beep(); // alert player
        string s =
          "Error: Minimum bet is " +
          machine.MinBet.ToString() +
          ".  Maximum bet is " +
          machine.MaxBet.ToString() + ".";
        showStatus(s);
      }
      return bet.Amount;
    }

    private void deal() {

      disableCommand("Dealing...");
      setBet();
      freezeBet();
      hideCards();


      // deal a hand...
      dealHand = machine.Deal();
      showCards(dealHand);

      // clear and enable the HOLD checkboxes...
      clearHoldCheckBoxes();
      enableHoldCheckBoxes();

      // tell player what to do...
      showMessage("Hold and Draw");
      showStatus("Hold cards and click the DRAW button.");
      enableCommand("&DRAW");
    }

    private void setBet() {
      int newBet = Int32.Parse(betTextBox.Text);
      Bet bet =
        new Bet(newBet,uiCredits, machine.MinBet, machine.MaxBet);
      uiBet = bet.Amount;
      uiCredits = bet.Credits;
      showMoney();
    }

    private void draw() {

      disableHoldCheckBoxes();
      disableCommand("Drawing...");

      // hold cards...
      string holdString = "";
      if (hold1.Checked) holdString += "1";
      if (hold2.Checked) holdString += "2";
      if (hold3.Checked) holdString += "3";
      if (hold4.Checked) holdString += "4";
      if (hold5.Checked) holdString += "5";

      drawHand = machine.Draw(dealHand, holdString, uiBet);

      // hide cards which have not been held...
      if (!hold1.Checked) hideCard(card1);
      if (!hold2.Checked) hideCard(card2);
      if (!hold3.Checked) hideCard(card3);
      if (!hold4.Checked) hideCard(card4);
      if (!hold5.Checked) hideCard(card5);
      pause(); // let the player see the backs of the cards

      showCards(drawHand);

      // update UI...
      int won = drawHand.Score * uiBet;
      uiCredits += won;
      showMoney();
      showMachineStatistics();
      showMessage(drawHand.Title);
      showStatus(drawHand.Title + "  -  Scores " + drawHand.Score);
      checkGameOver();
    }

    private void checkGameOver() {
      // check if player has enough money to go on...
      if (machine.MinBet > uiCredits) {
        disableCommand("Game Over");
        showStatus("Game over!");
        freezeBet();
        beep(); // alert player
      } else {
        enableCommand("&DEAL");
        focusCommand();
        unfreezeBet();
      }
    }

    private void newGame() {
      // start (again) with full credits...
      initPokerMachine();
      hideCards();
      clearHoldCheckBoxes();
      disableHoldCheckBoxes();
      unfreezeBet();
      showMachineStatistics();
      showMoney();
      enableCommand("&DEAL");
      focusCommand();
      showMessage("Click DEAL to Start");
      showStatus("Place Your Bet and Click DEAL to Start");
    }

    private void enableCommand(string s) {
      dealDrawButton.Text = s;
      dealDrawButton.Enabled = true;
      startOverButton.Enabled = true;
    }

    private void disableCommand(string s) {
      dealDrawButton.Enabled = false;
      dealDrawButton.Text = s;
      if (s.Equals("Game Over")) {
        startOverButton.Enabled = true;
        startOverMenuItem.Enabled = true;
      }
      else {
        startOverButton.Enabled = false;
        startOverMenuItem.Enabled = false;
      }
    }

    private void showMessage(string s) {
      messageLabel.Text = s;
    }

    private void showStatus(string s) {
      statusBarPanel.Text = s;
    }

    private void freezeBet() {
      betTextBox.ReadOnly = true;
    }

    private void unfreezeBet() {
      betTextBox.ReadOnly = false;
    }

    private void hideCards() {
      // display the backs of the cards...
      card1.Image = card2.Image = card3.Image =
        card4.Image = card5.Image = getImage("CB");
      Application.DoEvents();
    }

    private void hideCard(PictureBox card) {
      card.Image = getImage("CB");
    }

    private void showCards(Hand h) {
      card1.Image = getImage(h.CardName(1)); pause();
      card2.Image = getImage(h.CardName(2)); pause();
      card3.Image = getImage(h.CardName(3)); pause();
      card4.Image = getImage(h.CardName(4)); pause();
      card5.Image = getImage(h.CardName(5)); pause();
    }

    private void showMoney() {
      showCredits();
      showBet();
    }

    private void showCredits() {
      creditsLabel.Text = uiCredits.ToString();
    }

    private void showBet() {
      betTextBox.Text = uiBet.ToString();
    }

    private void showMachineStatistics() {
      machineStatsLabel.Text = machine.Stats;
    }

    private void clearHoldCheckBoxes() {
      hold1.Checked = hold2.Checked = hold3.Checked =
        hold4.Checked = hold5.Checked = false;
    }

    private void enableHoldCheckBoxes() {
      hold1.Enabled = hold2.Enabled = hold3.Enabled =
        hold4.Enabled = hold5.Enabled = true;
      hold1.Focus();
    }

    private void disableHoldCheckBoxes() {
      hold1.Enabled = hold2.Enabled = hold3.Enabled =
        hold4.Enabled = hold5.Enabled = false;
    }

    private void focusCommand() {
      dealDrawButton.Focus();
    }

    private Image getImage(string imgName) {
      string fileName = @"..\images\" + imgName + ".GIF";
      try {
        return Image.FromFile(fileName);
      } catch (Exception e) {
        MessageBox.Show(
          "Error loading card image file: " + e.Message,
          "Error!",
          MessageBoxButtons.OK, MessageBoxIcon.Error);
        return null;
      }
    }

    private Icon getIcon(string iconName) {
      string fileName = iconName + ".ICO";
      try {
        return new Icon(fileName);
      } catch (Exception e) {
        MessageBox.Show(
          "Error loading icon file: " + e.Message,
          "Error!",
          MessageBoxButtons.OK, MessageBoxIcon.Error);
        return null;
      }
    }

    private void pause() {
      pause(200);
    }

    private void pause(int n) {
      Application.DoEvents();
      Thread.Sleep(n);
    }

    private void beep() {
      MessageBeep(0);
    }

    // private form variables...
    private Machine machine;          // the poker machine
    private int uiCredits;            // amount of player credits
    private int uiBet;                // amount of player's bet
    private Hand dealHand;            // hand dealt
    private Hand drawHand;            // hand drawn
    private Button dealDrawButton;    // click to deal/draw
    private Button startOverButton;   // click to start over
    private Label messageLabel;       // informational message
    private Label creditsLabel;       // display credits remaining
    private Label machineStatsLabel;  // display mechine stats
    private TextBox betTextBox;       // input player bet

    // start over menu item...
    private MenuItem startOverMenuItem;

    // display card images...
    private PictureBox card1, card2, card3, card4, card5;

    // display checkbox underneath each card...
    private CheckBox hold1, hold2, hold3, hold4, hold5;

    // status bar display...
    private StatusBarPanel statusBarPanel;

    [DllImportAttribute("user32.dll")]
    public static extern int MessageBeep(int type); // error beep
  }
}
