// file         : xmlrep.cs
// description  : generate XML/HTML report for poker machine

namespace Poker {

  using System;
  using System.IO;
  using System.Xml;
  using System.Xml.Xsl;
  using System.Xml.XPath;
  using System.Xml.Serialization;
  using System.Diagnostics;

  [XmlRootAttribute]
  public class XmlRep {

    public XmlRep() {
      Bank b = new Bank();
      this.TakenIn = b.TakenIn;
      this.PaidOut = b.PaidOut;
      this.Profit = b.Profit;
      this.HouseMargin = Math.Round(b.HouseMargin, 2);
      this.Delta = Math.Round(b.Delta, 2);
      this.Bias = b.Bias;
    }

    [XmlElementAttribute]
    public int TakenIn;

    [XmlElementAttribute]
    public int PaidOut;

    [XmlElementAttribute]
    public int Profit;

    [XmlElementAttribute]
    public int Bias;

    [XmlElementAttribute]
    public Double HouseMargin;

    [XmlElementAttribute]
    public Double Delta;

    public static void Main() {

      Console.WriteLine("Serializing data to report.xml...");
      XmlSerializer sr = new XmlSerializer(typeof(XmlRep));
      TextWriter tw = new StreamWriter("report.xml");
      sr.Serialize(tw, new XmlRep());
      tw.Close();

      Console.WriteLine("Creating report.html...");
      XPathDocument xd =
        new XPathDocument(new XmlTextReader("Report.xml"));
      tw = new StreamWriter("Report.html");
      XslTransform xt = new XslTransform();
      xt.Load("pokrep.xsl");
      xt.Transform(xd, null, tw);
      tw.Close();

      Console.WriteLine("Launching report in browser...");
      ProcessStartInfo si = new ProcessStartInfo();
      si.FileName = "Report.html";
      si.Verb = "open";
      Process pr = new Process();
      pr.StartInfo = si;
      pr.Start();

      Console.WriteLine("Done!");
    }
  }
}