namespace Poker {

  using System;
  using System.Configuration;
  using System.IO;
  using System.Data;
  using System.Data.SqlClient;

  public class Bank {

    public Bank() {
      setConnectString();
      TargetMargin = GetParm("TargetMargin", 25);
      refresh();
    }

    public readonly int TargetMargin;

    public int TakenIn { get { return takenIn; } }
    public int PaidOut { get { return paidOut; } }
    public int Profit  { get { return profit;  } }

    public double HouseMargin { get {
      if (takenIn == 0) return TargetMargin;
      return (double) profit * 100.0 / takenIn;
    } }

    public double Delta { get {
      return HouseMargin - TargetMargin;
    } }

    public int Bias { get {
      if (Delta >= 0.0) return 0;
      int bias = (int) Math.Round(Math.Abs(Delta));
      if (bias > 10) return 10;
      return bias;
    } }

    public string Status { get {
      return status;
    } }

    public string Text { get {
        return "\n" +
        status + "\n" +
        "===========================\n" +
        "Taken In        : " + takenIn + "\n" +
        "Paid Out        : " + paidOut + "\n" +
        "Profit          : " + profit + "\n" +
        "House Margin %  : " +
          String.Format("{0:00.00}", HouseMargin) + "\n" +
        "Target Margin % : " +
          String.Format("{0:00.00}", TargetMargin) + "\n" +
        "Delta           : " +
          String.Format("{0:00.00}", Delta) + "\n" +
        "Bias            : " + Bias + "\n";
    } }

    public override string ToString() {
      return Text;
    }

    public void SaveGame(string hand, int score, int bet) {

      if (connectString == "") return;

      SqlConnection conn = null;
      try {
        conn = new SqlConnection(connectString);
      } catch (Exception e) {
        new MsgLog(String.Format( "Bank.SaveGame(): {0} - {1}",
                                  "Cannot create SqlConnection",
                                  e.Message));
        return;
      }

      string sql =
        "INSERT INTO games(hand, score, bet) VALUES " +
        "('" + hand + "'," + score + "," + bet + ")";
      SqlCommand comm = null;
      try {
        comm = new SqlCommand(sql, conn);
      } catch (Exception e) {
        new MsgLog(String.Format( "Bank.SaveGame(): {0} - {1}",
                                  "Cannot create SqlCommand",
                                  e.Message));
        return;
      }

      try {
        conn.Open();
      } catch (Exception e) {
        new MsgLog(String.Format( "Bank.SaveGame(): {0} - {1}",
                                  "Cannot open SqlConnection",
                                  e.Message));
        return;
      }

      try {
        comm.ExecuteNonQuery();
      } catch (Exception e) {
        new MsgLog(String.Format( "Bank.SaveGame(): {0} - {1}",
                                  "Cannot execute SqlCommand",
                                  e.Message));
        return;
      }
      finally {
        if (conn.State == ConnectionState.Open) conn.Close();
      }

      refresh();
    }

    public int GetParm(string parmName, int defaultValue) {

      int parmValue = defaultValue;

      if (connectString == "") return parmValue;

      string sql =
        "SELECT value FROM integers WHERE name='" + parmName + "'";
      DataSet ds = new DataSet("PokerParm");
      SqlDataAdapter sda = new SqlDataAdapter(sql, connectString);

      try {
        sda.Fill(ds, "result");
        parmValue = (int) ds.Tables["result"].Rows[0][0];
      } catch (Exception e) {
        connectString = "";
        new MsgLog(
          String.Format("Bank.GetParm(): {0}", e.Message));
      }
      return parmValue;
    }

    private void setConnectString() {
      connectString = ConfigurationSettings.AppSettings["dsn"];
      if (connectString == null) connectString = "";
      if (connectString == "")
        connectString =
         @"server=(local)\NetSDK;" +
         @"database=poker;trusted_connection=yes";
    }

    private void refresh() {

      if (connectString == "") return;

      string sql =
        "SELECT " +
          "SUM(bet) AS taken_in, " +
          "SUM(score * bet) AS paid_out, " +
          "SUM(bet) - SUM(score * bet) as profit " +
        "FROM games";

      SqlDataAdapter sda = null;
      try {
        sda = new SqlDataAdapter(sql, connectString);
        DataSet ds = new DataSet("PokerProfit");
        sda.Fill(ds, "stats");
        DataRow dr = ds.Tables[0].Rows[0];
        takenIn = (int) dr[0];
        paidOut = (int) dr[1];
        profit  = (int) dr[2];
        status = "Machine Stats (All Players)";
      } catch (Exception e) {
        new MsgLog(
          String.Format("Bank.refresh(): {0}", e.Message));
      }
    }

    // private static Bank bank = null;
    private string connectString = "";
    private string status = "Machine Stats Unavailable";
    private int    takenIn = 0;
    private int    paidOut = 0;
    private int    profit = 0;
  }
}