namespace Poker {

  using System;

  public class Machine {

    public readonly int MinBet;
    public readonly int MaxBet;
    public readonly int StartCredits;
    public readonly int Bias;

    // private constructor...
    private Machine() {
      bank = new Bank();
      MinBet = bank.GetParm("MinBet", 1);
      MaxBet = bank.GetParm("MaxBet", 5);
      StartCredits = bank.GetParm("StartCredits", 100);
      Bias = bank.Bias;
    }

    public static Machine Instance {
      get {
        // allow just one instance...
        if (machine == null) machine = new Machine();
        return machine;
      }
    }

    public Hand Deal() {
      Hand hand = new Hand();
      int bias = Bias;
      while (hand.Score > 0 && bias-- > 0)
        hand = new Hand();
      return hand;
    }

    public Hand Draw(Hand oldHand, string holdCards, int bet) {
      int bias = Bias;
      Hand newHand = new Hand(oldHand, holdCards);
      while (newHand.Score > 0 && bias-- > 0)
        newHand = new Hand(oldHand, holdCards);
      bank.SaveGame(newHand.ToString(), newHand.Score, bet);
      return newHand;
    }

    public Hand Draw(string handString, string holdCards, int bet) {
      return Draw(new Hand(handString), holdCards, bet);
    }

    public string Stats { get {
      return bank.Text;
    } }

    public static string PayoutTable { get {
        return "\n" +
        "Payout Table\n" +
        "============\n" +
        "Royal Flush     : 10\n" +
        "Straight Flush  :  9\n" +
        "Four of a Kind  :  8\n" +
        "Full House      :  7\n" +
        "Flush           :  6\n" +
        "Straight        :  5\n" +
        "Three of a Kind :  4\n" +
        "Two Pair        :  3\n" +
        "Jacks or Better :  2\n";
    } }

    private static Machine machine = null;
    private Bank bank = null;
  }
}