Option Strict On    ' no implicit type conversions
Option Explicit On  ' variables must be declared

Imports System
Imports Microsoft.VisualBasic.Constants
Imports System.Configuration
Imports System.Data
Imports System.Data.SqlClient

Namespace Poker

  Public Class Bank

    Public Sub New()
      setConnectString()
      TargetMargin = GetParm("TargetMargin", 25)
      refresh()
    End Sub

    Public ReadOnly TargetMargin As Integer

    Public ReadOnly Property TakenIn As Integer
      Get
        Return _takenIn
      End Get
    End Property

    Public ReadOnly Property PaidOut As Integer
      Get
        Return _paidOut
      End Get
    End Property

    Public ReadOnly Property Profit As Integer
      Get
        Return _profit
      End Get
    End Property

    Public ReadOnly Property HouseMargin As Double
      Get
        If _takenIn = 0 Then Return TargetMargin
        Return CDbl(_profit * 100.0 / _takenIn)
      End Get
    End Property

    Public ReadOnly Property Delta As Double
      Get
        Return HouseMargin - TargetMargin
      End Get
    End Property

    Public ReadOnly Property Bias As Integer
      Get
        If Delta >= 0.0 Then Return 0
        Dim _bias As Integer = CInt(Math.Round(Math.Abs(Delta)))
        If _bias > 10 Then Return 10
        Return _bias
      End Get
    End Property

    Public ReadOnly Property Status As String
      Get
        Return _status
      End Get
    End Property

    Public ReadOnly Property Text As String
      Get
        return vbCrLf & _
        _status & vbCrLf & _
        "===========================" & vbCrLf & _
        "Taken In        : " & _takenIn & vbCrLf & _
        "Paid Out        : " & _paidOut & vbCrLf & _
        "Profit          : " & _profit & vbCrLf & _
        "House Margin %  : " & _
          String.Format("{0:00.00}", HouseMargin) & vbCrLf & _
        "Target Margin % : " & _
          String.Format("{0:00.00}", TargetMargin) & vbCrLf & _
        "Delta           : " & _
          String.Format("{0:00.00}", Delta) & vbCrLf & _
        "Bias            : " & Bias & vbCrLf
      End Get
    End Property

    Overrides Public Function ToString() As String
      Return Text
    End Function

    Public Sub SaveGame(hand As String, score As Integer, bet As Integer)

      If _connectString = "" Then Exit Sub

      Dim conn As SqlConnection = Nothing
      Try
        conn = New SqlConnection(_connectString)
      Catch e As Exception
        MsgLog.Write(String.Format( "Bank.SaveGame(): {0} - {1}", _
                                  "Cannot create SqlConnection", _
                                  e.Message))
        Return
      End Try

      Dim sql As String = _
        "INSERT INTO games(hand, score, bet) VALUES " & _
        "('" & hand & "'," & score & "," & bet & ")"
      Dim comm As SqlCommand = Nothing
      Try
        comm = New SqlCommand(sql, conn)
      Catch e As Exception
        MsgLog.Write(String.Format( "Bank.SaveGame(): {0} - {1}", _
                                  "Cannot create SqlCommand", _
                                  e.Message))
        Return
      End Try

      Try
        conn.Open()
      Catch e As Exception
        MsgLog.Write(String.Format( "Bank.SaveGame(): {0} - {1}", _
                                  "Cannot open SqlConnection", _
                                  e.Message))
        Return
      End Try

      Try
        comm.ExecuteNonQuery()
      Catch e As Exception
        MsgLog.Write(String.Format( "Bank.SaveGame(): {0} - {1}", _
                                  "Cannot execute SqlCommand", _
                                  e.Message))
        Return
      Finally
        If conn.State = ConnectionState.Open Then conn.Close()
      End Try

      refresh()
    End Sub

    Public Function GetParm(parmName As String, defaultValue As Integer) As Integer

      Dim parmValue As Integer = defaultValue

      If _connectString = "" Then Return parmValue

      Dim sql As String = _
        "SELECT value FROM integers WHERE name='" & parmName & "'"
      Dim ds As DataSet = New DataSet("PokerParm")
      Dim sda As SqlDataAdapter = New SqlDataAdapter(sql, _connectString)

      Try
        sda.Fill(ds, "result")
        Dim dr As DataRow = ds.Tables("result").Rows(0)
        parmValue = CInt(dr(0))
      Catch e As Exception
        _connectString = ""
        MsgLog.Write(String.Format("Bank.GetParm(): {0}", e.Message))
      End Try
      Return parmValue
    End Function

    Private Sub setConnectString()
      _connectString = ConfigurationSettings.AppSettings("dsn")
      If _connectString = Nothing Then _connectString = ""
      If _connectString = "" Then
        _connectString = _
         "server=(local)\NetSDK;" & _
         "database=poker;trusted_connection=yes"
      End If
    End Sub

    Private Sub refresh()

      If _connectString = "" Then Return

      Dim sql As String = _
        "SELECT " & _
          "SUM(bet) AS taken_in, " & _
          "SUM(score * bet) AS paid_out, " & _
          "SUM(bet) - SUM(score * bet) as profit " & _
        "FROM games"

      Dim sda As SqlDataAdapter = Nothing
      Try
        sda = New SqlDataAdapter(sql, _connectString)
        Dim ds As DataSet = New DataSet("PokerProfit")
        sda.Fill(ds, "stats")
        Dim dr As DataRow = ds.Tables(0).Rows(0)
        _takenIn = CInt(dr(0))
        _paidOut = CInt(dr(1))
        _profit  = CInt(dr(2))
        _status = "Machine Stats (All Players)"
      Catch e As Exception
        MsgLog.Write( _
          String.Format("Bank.refresh(): {0}", e.Message))
      End Try
    End Sub

    ' private static Bank bank = null;
    Private _connectString As String = ""
    Private _status As String = "Machine Stats Unavailable"
    Private _takenIn As Integer = 0
    Private _paidOut As Integer = 0
    Private _profit As Integer = 0

  End Class
End Namespace
