Option Strict On    ' no implicit type conversions
Option Explicit On  ' variables must be declared

Imports System

Namespace Poker

  Public Class Hand

    Public Sub New()
      Me.New(New Random())
    End Sub

    Public Sub New(r as Random)

      cards(1) = New Card(r) ' deal first card

      ' deal remaining cards...
      Dim i As Integer
      For i = 2 To 5
        cards(i) = New Card(r)
        While containsCard(cards(i), cards, i - 1)
          cards(i) = New Card(r)
        End While
      Next

    End Sub

    Public Sub New(handText As String)
      cardsFromString(handText)
    End Sub

    Public Sub New(handText As String, holdString As String)
      cardsFromString(handText)
      holdCards(holdString)
      draw()
    End Sub

    Public Sub New(hand As Hand, holdString As String)
      Me.cards = hand.cards
      holdCards(holdString)
      draw()
    End Sub

    Public ReadOnly Property Score As Integer
      Get
        If _score < 0 Then calcScore()
        Return _score
      End Get
    End Property

    Public ReadOnly Property Title As String
      Get
        Return titles(Score)
      End Get
    End Property

    Public ReadOnly Property CardName(cardNum As Integer) As String
      Get
        Return cards(cardNum).Name
      End Get
    End Property

    Public ReadOnly Property Text As String
      Get
        return  CardName(1) + " " + _
                CardName(2) + " " + _
                CardName(3) + " " + _
                CardName(4) + " " + _
                CardName(5)
      End Get
    End Property

    Overrides Public Function ToString() As String
      return Text
    End Function

    Private Sub cardsFromString(handText As String)
      Dim delims() As Char = {" "C}
      Dim cardStrings() As String = handText.Split(delims)
      Dim i As Integer
      For i = 0 To cardStrings.Length - 1
        cards(i + 1) = New Card(cardStrings(i))
      Next
    End Sub

    Private Sub holdCards(holdString As String)
      Dim cardNum As Integer
      for cardNum = 1 To 5
        If holdString.IndexOf(cardNum.ToString()) >= 0 Then
          isHold(cardNum) = True
        End If
      Next
    End Sub

    Private Sub draw()

      ' remember which cards player has seen...
      Dim seen(10) As Card
      Dim i As Integer
      For i = 1 To 5
        seen(i) = cards(i)
      Next

      Dim numSeen As Integer = 5
      Dim r As Random = New Random()
      For i = 1 To 5
        If Not isHold(i) Then

          cards(i) = New Card(r)
          ' avoid duplicates...
          While containsCard(cards(i), seen, numSeen)
            cards(i) = New Card(r)
          End While

          ' remember we saw this card
          numSeen = numSeen + 1
          seen(numSeen) = cards(i)
        End If
      Next
    End Sub

    Private Function containsCard(c As Card, cs() As Card, count As Integer) As Boolean
      Dim i As Integer
      For i = 1 To count
        if c.Equals(cs(i)) Then
          Return True
        End If
      Next i
      Return False
    End Function

    Private Sub calcScore()

      ' are cards all of the same suit?
      Dim isFlush As Boolean = True
      Dim s As Integer = cards(1).Suit
      Dim i As Integer
      For i = 2 To 5
        If s <> cards(i).Suit Then
          isFlush = False
          Exit For
        End If
      Next

      ' sort card values...
      Dim sortedValues(5) As Integer
      sortedValues(0) = 0
      For i = 1 To 5
        sortedValues(i) = cards(i).Number
      Next
      Array.Sort(sortedValues)

      ' do we have a straight?
      Dim isStraight As Boolean = True
      For i = 1 To 4
        If (sortedValues(i) + 1) <> sortedValues(i+1) Then
          isStraight = False
          Exit For
        End If
      Next

      ' maybe it is a straight from the ace (i.e. A, 2, 3, 4, 5)
      If Not isStraight Then
        If sortedValues(1) = 2 And _
           sortedValues(2) = 3 And _
           sortedValues(3) = 4 And _
           sortedValues(4) = 5 And _
           sortedValues(5) = 14 Then ' ace on top
          isStraight = true
        End If
      End If

      ' is it a straight to the ace?
      Dim isTopStraight As Boolean = isStraight And sortedValues(5) = 14

      ' now calculate score...

      ' royal flush...
      If isTopStraight And isFlush Then
        _score = 10
        Return
      End If

      ' straight flush...
      If isStraight And isFlush Then
        _score = 9
        Return
      End If

      ' four of a kind...
      If sortedValues(1) = sortedValues(2) And _
         sortedValues(2) = sortedValues(3) And _
         sortedValues(3) = sortedValues(4) Then
        _score = 8
        Return
      End If
      if sortedValues(2) = sortedValues(3) And _
         sortedValues(3) = sortedValues(4) And _
         sortedValues(4) = sortedValues(5) Then
        _score = 8
        Return
      End If

      ' full house...
      if sortedValues(1) = sortedValues(2) And _
         sortedValues(2) = sortedValues(3) And _
         sortedValues(4) = sortedValues(5) Then
        _score = 7
        Return
      End If
      If sortedValues(1) = sortedValues(2) And _
         sortedValues(3) = sortedValues(4) And _
         sortedValues(4) = sortedValues(5) Then
        _score = 7
        Return
      End If

      ' flush...
      If isFlush Then
        _score = 6
        Return
      End If

      ' straight...
      If isStraight Then
        _score = 5
        Return
      End If

      ' three of a kind...
      If sortedValues(1) = sortedValues(2) And _
         sortedValues(2) = sortedValues(3) Then
        _score = 4
        Return
      End If
      If sortedValues(2) = sortedValues(3) And _
         sortedValues(3) = sortedValues(4) Then
        _score = 4
        Return
      End If
      If sortedValues(3) = sortedValues(4) And _
         sortedValues(4) = sortedValues(5) Then
        _score = 4
        Return
      End If

      ' two pair...
      If sortedValues(1) = sortedValues(2) And _
         sortedValues(3) = sortedValues(4) Then
        _score = 3
        Return
      End If
      If sortedValues(1) = sortedValues(2) And _
         sortedValues(4) = sortedValues(5) Then
        _score = 3
        Return
      End If
      If sortedValues(2) = sortedValues(3) And _
         sortedValues(4) = sortedValues(5) Then
        _score = 3
        Return
      End If

      ' jacks or better...
      If sortedValues(1) > 10 And _
         sortedValues(1) = sortedValues(2) Then
        _score = 2
        Return
      End If
      If sortedValues(2) > 10 And _
         sortedValues(2) = sortedValues(3) Then
        _score = 2
        Return
      End If
      If sortedValues(3) > 10 And _
         sortedValues(3) = sortedValues(4) Then
        _score = 2
        Return
      End If
      If sortedValues(4) > 10 And _
         sortedValues(4) = sortedValues(5) Then
        _score = 2
        Return
      End If
      _score = 0

    End Sub

    Private cards() as Card = _
      {Nothing, Nothing, Nothing, Nothing, Nothing, Nothing}
    Private isHold() As Boolean = _
      {False, False, False, False, False, False}
    Private Shared titles() As String = { _
      "No Score", _
      "", _
      "Jacks or Better", _
      "Two Pair", _
      "Three of a Kind", _
      "Straight", _
      "Flush", _
      "Full House", _
      "Four of a Kind", _
      "Straight Flush", _
      "Royal Flush"  _
    }

    Private _score As Integer = -1
  End Class
End Namespace
