Option Strict On    ' no implicit type conversions
Option Explicit On  ' variables must be declared

Imports System
Imports Microsoft.VisualBasic.Constants

Namespace Poker

  Public Class Machine

    Public ReadOnly MinBet As Integer
    Public ReadOnly MaxBet As Integer
    Public ReadOnly StartCredits As Integer
    Public ReadOnly Bias As Integer

    ' private constructor...
    Private Sub New()
      bank = New Bank()
      MinBet = bank.GetParm("MinBet", 1)
      MaxBet = bank.GetParm("MaxBet", 5)
      StartCredits = bank.GetParm("StartCredits", 100)
      Bias = bank.Bias
    End Sub

    Public ReadOnly Shared Property Instance As Machine
      Get
        ' allow just one instance...
        If machine Is Nothing Then machine = New Machine()
        Return machine
      End Get
    End Property

    Public Function Deal() As Hand
      Dim h As Hand = New Hand()
      Dim b As Integer = Bias
      While h.Score > 0 And b > 0
        b = b - 1
        h = new Hand()
      End While
      Return h
    End Function

    Public Function Draw(oldHand As Hand, holdCards As String, bet As Integer) As Hand
      Dim b As Integer = Bias
      Dim newHand As Hand = new Hand(oldHand, holdCards)
      While newHand.Score > 0 And b > 0
        b = b - 1
        newHand = new Hand(oldHand, holdCards)
      End While
      bank.SaveGame(newHand.ToString(), newHand.Score, bet)
      Return newHand
    End Function

    Public Function Draw(handString As String, holdCards As String, bet As Integer) As Hand
      Return Draw(new Hand(handString), holdCards, bet)
    End Function

    Public ReadOnly Property Stats As String
      Get
        Return bank.Text
      End Get
    End Property

    Public ReadOnly Shared Property PayoutTable As String
      Get
        Return vbCrLf & _
        "Payout Table" & vbCrLf & _
        "============" & vbCrLf & _
        "Royal Flush     : 10" & vbCrLf & _
        "Straight Flush  :  9" & vbCrLf & _
        "Four of a Kind  :  8" & vbCrLf & _
        "Full House      :  7" & vbCrLf & _
        "Flush           :  6" & vbCrLf & _
        "Straight        :  5" & vbCrLf & _
        "Three of a Kind :  4" & vbCrLf & _
        "Two Pair        :  3" & vbCrLf & _
        "Jacks or Better :  2" & vbCrLf
      End Get
    End Property

    Private Shared machine As Machine = Nothing
    Private bank As Bank = Nothing

  End Class
End Namespace
